@@ -13,6 +13,7 @@
 #include <assert.h>
 #include "mallocvar.h"
 #include "pbm.h"
+#include "pm_c_util.h"
 
 /*
   The algorithm is based on describing the 2 or 8 pixels in a cell with a
@@ -83,15 +84,15 @@ BSQ,D04, D04,D05, D04,'L', D05,'[', '<','Z', '/','Z', 'c','k',D06,'R',/*D0-DF*/
 
 
 static void
-makeRowOfSigs(FILE *         const ifP,
-              unsigned int   const cols,
-              unsigned int   const rows,
-              int            const format,
-              unsigned int   const cellWidth,
-              unsigned int   const cellHeight,
-              unsigned int   const row,
-              unsigned int * const sig,
-              unsigned int   const ccols) {
+makeRowOfSignatures(FILE *         const ifP,
+                    unsigned int   const cols,
+                    unsigned int   const rows,
+                    int            const format,
+                    unsigned int   const cellWidth,
+                    unsigned int   const cellHeight,
+                    unsigned int   const row,
+                    unsigned int * const sig,
+                    unsigned int   const ccols) {
 /*----------------------------------------------------------------------------
    Compute the signatures for every cell in a row.
 
@@ -183,23 +184,30 @@ pbmtoascii(FILE *       const ifP,
            unsigned int const cellHeight,
            const char * const carr) {
 
-    int cols, rows, format;
+    int format;
+    int cols, rows;
+        /* Dimensions of the input in pixels */
     unsigned int ccols;
+        /* Width of the output in characters */
     char * line;         /* malloc'ed array */
     unsigned int row;
-    unsigned int * sig;  /* malloc'ed array */
-        /* This describes in a single integer the pixels of a cell,
-           as described above.
+    unsigned int * signatureRow;  /* malloc'ed array */
+        /* This is the cell signatures of a row of cells.
+           signatureRow[0] is the signature for the first (leftmost) cell
+           in the row, signatureRow[1] is the signature for the next one,
+           etc.  A signature is an encoding of the pixels of a cell as
+           an integer, as described above.
         */
-    assert(cellWidth * cellHeight <= sizeof(sig[0]*8));
+    assert(cellWidth * cellHeight <= sizeof(signatureRow[0])*8);
 
     pbm_readpbminit(ifP, &cols, &rows, &format);
 
     ccols = (cols + cellWidth - 1) / cellWidth;
 
-    MALLOCARRAY(sig, ccols);
-    if (sig == NULL)
+    MALLOCARRAY(signatureRow, ccols);
+    if (signatureRow == NULL)
         pm_error("No memory for %u columns", ccols);
+
     MALLOCARRAY_NOFAIL(line, ccols+1);
     if (line == NULL)
         pm_error("No memory for %u columns", ccols);
@@ -207,16 +215,16 @@ pbmtoascii(FILE *       const ifP,
     for (row = 0; row < rows; row += cellHeight) {
         unsigned int endCol;
 
-        makeRowOfSigs(ifP, cols, rows, format, cellWidth, cellHeight,
-                      row, sig, ccols);
+        makeRowOfSignatures(ifP, cols, rows, format, cellWidth, cellHeight,
+                            row, signatureRow, ccols);
 
-        findRightMargin(sig, ccols, carr, &endCol);
+        findRightMargin(signatureRow, ccols, carr, &endCol);
 
-        assembleCellRow(sig, endCol, carr, line);
+        assembleCellRow(signatureRow, endCol, carr, line);
 
         puts(line);
     }
-    free(sig);
+    free(signatureRow);
     free(line);
 }
 
@@ -229,6 +237,8 @@ main(int argc, const char ** argv) {
     int argn, gridx, gridy;
     const char * carr;
     const char* usage = "[-1x2|-2x4] [pbmfile]";
+    bool carr1x2Spec = false; /* Initial value */
+    bool carr2x4Spec = false; /* Initial value */
 
     pm_proginit(&argc, argv);
 
@@ -236,25 +246,18 @@ main(int argc, const char ** argv) {
     argn = 1;
     gridx = 1;
     gridy = 2;
-    carr = carr1x2;
 
     /* Check for flags. */
-    while ( argn < argc && argv[argn][0] == '-' && argv[argn][1] != '\0' )
-    {
+    while ( argn < argc && argv[argn][0] == '-' && argv[argn][1] != '\0' ) {
         if ( pm_keymatch( argv[argn], "-1x2", 2 ) )
-        {
-            gridx = 1;
-            gridy = 2;
-            carr = carr1x2;
-        }
+            carr1x2Spec = true;
+
         else if ( pm_keymatch( argv[argn], "-2x4", 2 ) )
-        {
-            gridx = 2;
-            gridy = 4;
-            carr = carr2x4;
-        }
+            carr2x4Spec = true;
+
         else
             pm_usage( usage );
+
         ++argn;
     }
 
@@ -269,6 +272,20 @@ main(int argc, const char ** argv) {
     if ( argn != argc )
         pm_usage( usage );
 
+    if ( carr1x2Spec == true && carr2x4Spec == true) {
+            pm_error("You cannot specify both -1x2 and -2x4");
+    }
+    else if (carr2x4Spec == true) {
+            gridx = 2;
+            gridy = 4;
+            carr = carr2x4;
+    }
+    else {
+            gridx = 1;
+            gridy = 2;
+            carr = carr1x2;
+    }
+
     pbmtoascii(ifP, gridx, gridy, carr);
 
     pm_close(ifP);
